@@ -26,13 +26,12 @@
 
 #include <unicode/uchar.h>
 
-using namespace std;
 using namespace icu;
 
 Alphabet::Alphabet()
 {
-  spair[pair<int32_t, int32_t>(0,0)] = 0;
-  spairinv.push_back(pair<int32_t, int32_t>(0,0));
+  spair[std::pair<int32_t, int32_t>(0,0)] = 0;
+  spairinv.push_back(std::pair<int32_t, int32_t>(0,0));
 }
 
 Alphabet::~Alphabet()
@@ -71,20 +70,21 @@ Alphabet::copy(Alphabet const &a)
 }
 
 void
-Alphabet::includeSymbol(UString const &s)
+Alphabet::includeSymbol(UStringView s)
 {
   if(slexic.find(s) == slexic.end())
   {
     int32_t slexic_size = slexic.size();
-    slexic[s] = -(slexic_size+1);
-    slexicinv.push_back(s);
+    UString st{s};
+    slexic[st] = -(slexic_size+1);
+    slexicinv.push_back(st);
   }
 }
 
 int32_t
 Alphabet::operator()(int32_t const c1, int32_t const c2)
 {
-  auto tmp = make_pair(c1, c2);
+  auto tmp = std::make_pair(c1, c2);
   if(spair.find(tmp) == spair.end())
   {
     int32_t spair_size = spair.size();
@@ -96,13 +96,18 @@ Alphabet::operator()(int32_t const c1, int32_t const c2)
 }
 
 int32_t
-Alphabet::operator()(UString const &s)
+Alphabet::operator()(UStringView s)
 {
-  return slexic[s];
+  // While the documentation says this assumes existence, there are clearly code paths that call it with an unknown symbol and thus get 0 back AND create an entry for that 0. Changing it to just return 0 still passes all tests.
+  auto it = slexic.find(s);
+  if (it == slexic.end()) {
+    return 0;
+  }
+  return it->second;
 }
 
 int32_t
-Alphabet::operator()(UString const &s) const
+Alphabet::operator()(UStringView s) const
 {
   auto it = slexic.find(s);
   if (it == slexic.end()) {
@@ -112,7 +117,7 @@ Alphabet::operator()(UString const &s) const
 }
 
 bool
-Alphabet::isSymbolDefined(UString const &s)
+Alphabet::isSymbolDefined(UStringView s) const
 {
   return slexic.find(s) != slexic.end();
 }
@@ -124,7 +129,7 @@ Alphabet::size() const
 }
 
 void
-Alphabet::write(FILE *output)
+Alphabet::write(FILE *output) const
 {
   // First, we write the taglist
   Compression::multibyte_write(slexicinv.size(), output);  // taglist size
@@ -153,7 +158,7 @@ Alphabet::read(FILE *input)
 
   // Reading of taglist
   int32_t tam = Compression::multibyte_read(input);
-  map<int32_t, string> tmp;
+  std::map<int32_t, std::string> tmp;
   while(tam > 0)
   {
     tam--;
@@ -172,7 +177,7 @@ Alphabet::read(FILE *input)
     tam--;
     int32_t first = Compression::multibyte_read(input);
     int32_t second = Compression::multibyte_read(input);
-    pair<int32_t, int32_t> tmp(first - bias, second - bias);
+    std::pair<int32_t, int32_t> tmp(first - bias, second - bias);
     int32_t spair_size = a_new.spair.size();
     a_new.spair[tmp] = spair_size;
     a_new.spairinv.push_back(tmp);
@@ -184,8 +189,8 @@ Alphabet::read(FILE *input)
 void
 Alphabet::serialise(std::ostream &serialised) const
 {
-  Serialiser<const vector<UString> >::serialise(slexicinv, serialised);
-  Serialiser<vector<pair<int32_t, int32_t> > >::serialise(spairinv, serialised);
+  Serialiser<const std::vector<UString> >::serialise(slexicinv, serialised);
+  Serialiser<std::vector<std::pair<int32_t, int32_t> > >::serialise(spairinv, serialised);
 }
 
 void
@@ -195,11 +200,11 @@ Alphabet::deserialise(std::istream &serialised)
   slexic.clear();
   spairinv.clear();
   spair.clear();
-  slexicinv = Deserialiser<vector<UString> >::deserialise(serialised);
+  slexicinv = Deserialiser<std::vector<UString> >::deserialise(serialised);
   for (size_t i = 0; i < slexicinv.size(); i++) {
     slexic[slexicinv[i]] = -i - 1; // ToDo: This does not turn the result negative due to unsigned semantics
   }
-  spairinv = Deserialiser<vector<pair<int32_t, int32_t> > >::deserialise(serialised);
+  spairinv = Deserialiser<std::vector<std::pair<int32_t, int32_t> > >::deserialise(serialised);
   for (size_t i = 0; i < slexicinv.size(); i++) {
     spair[spairinv[i]] = i;
   }
@@ -220,7 +225,7 @@ Alphabet::writeSymbol(int32_t const symbol, UFILE *output) const
 }
 
 void
-Alphabet::getSymbol(UString &result, int32_t const symbol, bool uppercase) const
+Alphabet::getSymbol(UString &result, int32_t symbol, bool uppercase) const
 {
   if (symbol == 0) {
     return;
@@ -234,20 +239,20 @@ Alphabet::getSymbol(UString &result, int32_t const symbol, bool uppercase) const
 }
 
 bool
-Alphabet::isTag(int32_t const symbol) const
+Alphabet::isTag(int32_t symbol) const
 {
   return symbol < 0;
 }
 
-pair<int32_t, int32_t> const &
-Alphabet::decode(int32_t const code) const
+std::pair<int32_t, int32_t> const &
+Alphabet::decode(int32_t code) const
 {
   return spairinv[code];
 }
 
-set<int32_t>
+std::set<int32_t>
 Alphabet::symbolsWhereLeftIs(UChar32 l) const {
-  set<int32_t> eps;
+  std::set<int32_t> eps;
   for(const auto& sp: spair) {  // [(l, r) : tag]
     if(sp.first.first == l) {
       eps.insert(sp.second);
@@ -256,17 +261,17 @@ Alphabet::symbolsWhereLeftIs(UChar32 l) const {
   return eps;
 }
 
-void Alphabet::setSymbol(int32_t symbol, UString newSymbolString) {
+void Alphabet::setSymbol(int32_t symbol, UStringView newSymbolString) {
   //Should be a special character!
   if (symbol < 0) slexicinv[-symbol-1] = newSymbolString;
 }
 
 void
-Alphabet::createLoopbackSymbols(set<int32_t> &symbols, Alphabet &basis, Side s, bool nonTagsToo)
+Alphabet::createLoopbackSymbols(std::set<int32_t> &symbols, const Alphabet &basis, Side s, bool nonTagsToo)
 {
   // Non-tag letters get the same int32_t in spairinv across alphabets,
   // but tags may differ, so do those separately afterwards.
-  set<int32_t> tags;
+  std::set<int32_t> tags;
   for(auto& it : basis.spairinv)
   {
     if(s == left) {
@@ -301,3 +306,49 @@ Alphabet::createLoopbackSymbols(set<int32_t> &symbols, Alphabet &basis, Side s,
     }
   }
 }
+
+std::vector<int32_t>
+Alphabet::tokenize(UStringView str) const
+{
+  std::vector<int32_t> ret;
+  size_t end = str.size();
+  size_t i = 0;
+  UChar32 c;
+  while (i < end) {
+    U16_NEXT(str.data(), i, end, c);
+    if (c == '\\') {
+    } else if (c == '<') {
+      size_t j = i;
+      while (c != '>' && j < end) {
+        U16_NEXT(str.data(), j, end, c);
+      }
+      if (c == '>') {
+        ret.push_back(operator()(str.substr(i-1, j-i+1)));
+        i = j;
+      }
+    } else {
+      ret.push_back(static_cast<int32_t>(c));
+    }
+  }
+  return ret;
+}
+
+bool
+Alphabet::sameSymbol(int32_t tsym, const Alphabet& other, int32_t osym,
+                     bool allow_anys) const
+{
+  // if it's a letter, then it's equal across alphabets
+  if (tsym >= 0 && tsym == osym) return true;
+  if (tsym < 0 && osym < 0 &&
+      this->slexicinv[-tsym-1] == other.slexicinv[-osym-1]) {
+    return true;
+  }
+  if (allow_anys &&
+      ((tsym < 0 && this->slexicinv[-tsym-1] == u"<ANY_CHAR>"_uv && osym > 0) ||
+       (tsym < 0 && this->slexicinv[-tsym-1] == u"<ANY_TAG>"_uv && osym < 0) ||
+       (osym < 0 && other.slexicinv[-osym-1] == u"<ANY_CHAR>"_uv && tsym > 0) ||
+       (osym < 0 && other.slexicinv[-osym-1] == u"<ANY_TAG>"_uv && tsym < 0))) {
+    return true;
+  }
+  return false;
+}
