@@ -28,14 +28,13 @@
 #include <lttoolbox/input_file.h>
 #include <libxml/xmlreader.h>
 
+#include <deque>
 #include <map>
 #include <queue>
 #include <set>
 #include <string>
 #include <cstdint>
 
-using namespace std;
-
 /**
  * Kind of output of the generator module
  */
@@ -58,12 +57,7 @@ private:
   /**
    * Transducers in FSTP
    */
-  map<UString, TransExe> transducers;
-
-  /**
-   * Current state of lexical analysis
-   */
-  State current_state;
+  std::map<UString, TransExe> transducers;
 
   /**
    * Initial state of every token
@@ -78,57 +72,59 @@ private:
   /**
    * The final states of inconditional sections in the dictionaries
    */
-  map<Node *, double> inconditional;
+  std::map<Node *, double> inconditional;
 
   /**
    * The final states of standard sections in the dictionaries
    */
-  map<Node *, double> standard;
+  std::map<Node *, double> standard;
 
   /**
    * The final states of postblank sections in the dictionaries
    */
-  map<Node *, double> postblank;
+  std::map<Node *, double> postblank;
 
   /**
    * The final states of preblank sections in the dictionaries
    */
-  map<Node *, double> preblank;
+  std::map<Node *, double> preblank;
 
   /**
    * Merge of 'inconditional', 'standard', 'postblank' and 'preblank' sets
    */
-  map<Node *, double> all_finals;
+  std::map<Node *, double> all_finals;
 
   /**
    * Queue of blanks, used in reading methods
    */
-  queue<UString> blankqueue;
+  std::queue<UString> blankqueue;
 
   /**
    * Queue of wordbound blanks, used in reading methods
    */
-  queue<UString> wblankqueue;
+  std::deque<UString> wblankqueue;
+
+  std::deque<std::vector<int32_t>> transliteration_queue;
 
   /**
    * Set of characters being considered alphabetics
    */
-  set<UChar32> alphabetic_chars;
+  std::set<UChar32> alphabetic_chars;
 
   /**
    * Set of characters to escape with a backslash
    */
-  set<UChar32> escaped_chars;
+  std::set<UChar32> escaped_chars;
 
   /**
    * Set of characters to ignore
    */
-  set<UChar32> ignored_chars;
+  std::set<UChar32> ignored_chars;
 
   /**
    * Mapping of characters for simplistic diacritic restoration specified in RCX files
    */
-  map<int, set<int> > rcx_map;
+  std::map<int, std::set<int> > rcx_map;
 
   /**
    * Original char being restored
@@ -234,20 +230,7 @@ private:
    */
   int maxAnalyses = INT_MAX;
 
-  /**
-   * True if a wblank block ([[..]]xyz[[/]]) was just read
-   */
-  bool is_wblank;
-
-  /**
-   * True if skip_mode is false and need to collect wblanks
-   */
-  bool collect_wblanks;
-
-  /**
-   * True if a wblank has been processed for postgen and we need an ending wblank
-   */
-  bool need_end_wblank;
+  bool transliteration_drop_tilde = false;
 
   /**
    * Output no more than 'N' best weight classes
@@ -259,14 +242,6 @@ private:
    */
   void streamError();
 
-  /**
-   * Reads a wordbound blank (opening blank to closing blank) from the stream input -> [[...]]xyz[[/]]
-   * @param input the stream being read
-   * @param output the stream to write on
-   * @return true if the word enclosed by the wordbound blank has a ~ for postgeneration activation
-   */
-  bool wblankPostGen(InputFile& input, UFILE *output);
-
   /**
    * Returns true if the character code is identified as alphabetic
    * @param c the code provided by the user
@@ -296,13 +271,8 @@ private:
    */
   int readDecomposition(InputFile& input, UFILE *output);
 
-  /**
-   * Read text from stream (postgeneration version)
-   * @param input the stream to read
-   * @param output the stream to write on
-   * @return the next symbol in the stream
-   */
-  int readPostgeneration(InputFile& input, UFILE *output);
+  bool readTransliterationBlank(InputFile& input);
+  bool readTransliterationWord(InputFile& input);
 
   /**
    * Read text from stream (generation version)
@@ -318,7 +288,7 @@ private:
    * @param output the stream to write on
    * @return the queue of 0-symbols, and the next symbol in the stream
    */
-  pair<UString, int> readBilingual(InputFile& input, UFILE *output);
+  std::pair<UString, int> readBilingual(InputFile& input, UFILE *output);
 
   /**
    * Read text from stream (SAO version)
@@ -333,26 +303,6 @@ private:
    */
   void flushBlanks(UFILE *output);
 
-  /**
-   * Flush all the wordbound blanks remaining in the current process
-   * @param output stream to write blanks
-   */
-  void flushWblanks(UFILE *output);
-
-  /**
-   * Combine wordbound blanks in the queue and return them.
-   *
-   * May pop from 'wblankqueue' and set 'need_end_wblank' to true.
-   *
-   * If 'wblankqueue' (see which) is empty, we get an empty string,
-   * otherwise we return a semicolon-separated combination of opening
-   * wblanks in the queue. If there is only a closing wblank, we just
-   * set need_end_wblank.
-   *
-   * @return final wblank string
-  */
-  UString combineWblanks();
-
   /**
    * Calculate the initial state of parsing
    */
@@ -367,14 +317,14 @@ private:
    * Shortcut for filtering on all final states with current settings
    * Assumes that casefrom is non-empty
    */
-  UString filterFinals(const State& state, const UString& casefrom);
+  UString filterFinals(const State& state, UStringView casefrom);
 
   /**
    * Write a string to an output stream,
    * @param str the string to write, escaping characters
    * @param output the stream to write in
    */
-  void writeEscaped(UString const &str, UFILE *output);
+  void writeEscaped(UStringView str, UFILE *output);
 
   /**
    * Write a string to an output stream.
@@ -385,7 +335,7 @@ private:
    * @param output the stream to write in
    * @return how many blanks to pop and print after printing lu
    */
-  size_t writeEscapedPopBlanks(UString const &str, UFILE *output);
+  size_t writeEscapedPopBlanks(UStringView str, UFILE *output);
 
   /**
    * Write a string to an output stream, escaping all escapable characters
@@ -393,16 +343,7 @@ private:
    * @param str the string to write, escaping characters
    * @param output the stream to write in
    */
-  void writeEscapedWithTags(UString const &str, UFILE *output);
-
-
-  /**
-   * Checks if an string ends with a particular suffix
-   * @param str the string to test
-   * @param the searched suffix
-   * @returns true if 'str' has the suffix 'suffix'
-   */
-  static bool endsWith(UString const &str, UString const &suffix);
+  void writeEscapedWithTags(UStringView str, UFILE *output);
 
   /**
    * Prints a word
@@ -410,7 +351,7 @@ private:
    * @param lf lexical form of the word
    * @param output stream where the word is written
    */
-  void printWord(UString const &sf, UString const &lf, UFILE *output);
+  void printWord(UStringView sf, UStringView lf, UFILE *output);
 
   /**
    * Prints a word.
@@ -420,7 +361,7 @@ private:
    * @param lf lexical form of the word
    * @param output stream where the word is written
    */
-  void printWordPopBlank(UString const &sf, UString const &lf, UFILE *output);
+  void printWordPopBlank(UStringView sf, UStringView lf, UFILE *output);
 
   /**
    * Prints a word (Bilingual version)
@@ -428,7 +369,7 @@ private:
    * @param lf lexical form of the word
    * @param output stream where the word is written
    */
-  void printWordBilingual(UString const &sf, UString const &lf, UFILE *output);
+  void printWordBilingual(UStringView sf, UStringView lf, UFILE *output);
 
 
   /**
@@ -436,21 +377,21 @@ private:
    * @param lf lexical form
    * @param output stream where the word is written
    */
-  void printSAOWord(UString const &lf, UFILE *output);
+  void printSAOWord(UStringView lf, UFILE *output);
 
   /**
    * Prints an unknown word
    * @param sf surface form of the word
    * @param output stream where the word is written
    */
-  void printUnknownWord(UString const &sf, UFILE *output);
+  void printUnknownWord(UStringView sf, UFILE *output);
 
   void initDecompositionSymbols();
 
-  vector<UString> numbers;
+  std::vector<UString> numbers;
   int readTMAnalysis(InputFile& input);
 
-  unsigned int lastBlank(UString const &str);
+  unsigned int lastBlank(UStringView str);
 
   /**
    * Print one blankqueue item if there is one, or a given "space" value.
@@ -458,28 +399,37 @@ private:
    * @param val the space character to use if no blank queue
    * @param output stream where the word is written
    */
-  void printSpace(UChar32 const val, UFILE *output);
+  void printSpace(UChar32 val, UFILE *output);
   /**
    * Print one possibly escaped character
    * if it's a space and the blank queue is non-empty,
    * pop the first blank and print that instead
    */
-  void printChar(const UChar32 val, UFILE* output);
+  void printChar(UChar32 val, UFILE* output);
 
-  void skipUntil(InputFile& input, UFILE *output, UChar32 const character);
-  static UString removeTags(UString const &str);
+  void skipUntil(InputFile& input, UFILE *output, UChar32 character);
+  static UStringView removeTags(UStringView str);
   UString compoundAnalysis(UString str);
-  size_t firstNotAlpha(UString const &sf);
+
+  struct Indices {
+        size_t i_codepoint;
+        size_t i_utf16; // always >= i_codepoint since some codepoints take up 2 UTF-16's
+  };
+
+  /*
+   * Iterates through unicode characters, returns a Unicode character
+   * index and UTF-16 string index of first non-alphabetic character,
+   * or size of the string (in characters, string size)
+   *
+   * @return index of first non-alpha char, or string size, as a tuple of number of characters and index in string
+   */
+  Indices firstNotAlpha(UStringView sf);
 
   void analysis_wrapper_null_flush(InputFile& input, UFILE *output);
   void bilingual_wrapper_null_flush(InputFile& input, UFILE *output, GenerationMode mode = gm_unknown);
   void generation_wrapper_null_flush(InputFile& input, UFILE *output,
                                      GenerationMode mode);
-  void postgeneration_wrapper_null_flush(InputFile& input, UFILE *output);
-  void intergeneration_wrapper_null_flush(InputFile& input, UFILE *output);
-  void transliteration_wrapper_null_flush(InputFile& input, UFILE *output);
-
-  UString compose(UString const &lexforms, UString const &queue) const;
+  UString compose(UStringView lexforms, UStringView queue) const;
 
   void procNodeICX();
   void procNodeRCX();
@@ -488,21 +438,32 @@ private:
   bool isLastBlankTM = false;
 
   xmlTextReaderPtr reader;
+
+  static constexpr size_t max_case_insensitive_state_size = 65536;
+  /*
+   * Including lowercased versions for every character can potentially create very large states
+   * (See https://github.com/apertium/lttoolbox/issues/167 ). As a sanity-check we don't do
+   * case-insensitive matching if the state size exceeds max_case_insensitive_state_size.
+   *
+   * @return running with --case-sensitive or state size exceeds max
+   */
+  bool beCaseSensitive(const State& state) {
+    return caseSensitive || state.size() >= max_case_insensitive_state_size;
+  }
+
 public:
 
   /*
    * String constants
    */
-  static UString const XML_TEXT_NODE;
-  static UString const XML_COMMENT_NODE;
-  static UString const XML_IGNORED_CHARS_ELEM;
-  static UString const XML_RESTORE_CHAR_ELEM;
-  static UString const XML_RESTORE_CHARS_ELEM;
-  static UString const XML_VALUE_ATTR;
-  static UString const XML_CHAR_ELEM;
-  static UString const WBLANK_START;
-  static UString const WBLANK_END;
-  static UString const WBLANK_FINAL;
+  static constexpr UStringView XML_TEXT_NODE           = u"#text";
+  static constexpr UStringView XML_COMMENT_NODE        = u"#comment";
+  static constexpr UStringView XML_IGNORED_CHARS_ELEM  = u"ignored-chars";
+  static constexpr UStringView XML_RESTORE_CHAR_ELEM   = u"restore-char";
+  static constexpr UStringView XML_RESTORE_CHARS_ELEM  = u"restore-chars";
+  static constexpr UStringView XML_VALUE_ATTR          = u"value";
+  static constexpr UStringView XML_CHAR_ELEM           = u"char";
+  static constexpr UStringView WBLANK_FINAL            = u"[[/]]";
 
   FSTProcessor();
 
@@ -510,7 +471,8 @@ public:
   void initTMAnalysis();
   void initSAO(){initAnalysis();};
   void initGeneration();
-  void initPostgeneration();
+  void initPostgeneration(){initTransliteration();};
+  void initTransliteration();
   void initBiltrans();
   void initDecomposition();
 
@@ -520,29 +482,29 @@ public:
   void postgeneration(InputFile& input, UFILE *output);
   void intergeneration(InputFile& input, UFILE *output);
   void transliteration(InputFile& input, UFILE *output);
-  UString biltrans(UString const &input_word, bool with_delim = true);
-  UString biltransfull(UString const &input_word, bool with_delim = true);
+  UString biltrans(UStringView input_word, bool with_delim = true);
+  UString biltransfull(UStringView input_word, bool with_delim = true);
   void bilingual(InputFile& input, UFILE *output, GenerationMode mode = gm_unknown);
-  pair<UString, int> biltransWithQueue(UString const &input_word, bool with_delim = true);
-  UString biltransWithoutQueue(UString const &input_word, bool with_delim = true);
+  std::pair<UString, int> biltransWithQueue(UStringView input_word, bool with_delim = true);
+  UString biltransWithoutQueue(UStringView input_word, bool with_delim = true);
   void SAO(InputFile& input, UFILE *output);
-  void parseICX(string const &file);
-  void parseRCX(string const &file);
+  void parseICX(std::string const &file);
+  void parseRCX(std::string const &file);
 
   void load(FILE *input);
 
   bool valid() const;
 
-  void setCaseSensitiveMode(bool const value);
-  void setDictionaryCaseMode(bool const value);
-  void setBiltransSurfaceForms(bool const value);
-  void setIgnoredChars(bool const value);
-  void setRestoreChars(bool const value);
-  void setNullFlush(bool const value);
-  void setUseDefaultIgnoredChars(bool const value);
-  void setDisplayWeightsMode(bool const value);
-  void setMaxAnalysesValue(int const value);
-  void setMaxWeightClassesValue(int const value);
+  void setCaseSensitiveMode(bool value);
+  void setDictionaryCaseMode(bool value);
+  void setBiltransSurfaceForms(bool value);
+  void setIgnoredChars(bool value);
+  void setRestoreChars(bool value);
+  void setNullFlush(bool value);
+  void setUseDefaultIgnoredChars(bool value);
+  void setDisplayWeightsMode(bool value);
+  void setMaxAnalysesValue(int value);
+  void setMaxWeightClassesValue(int value);
   bool getNullFlush();
   bool getDecompoundingMode();
 };
