@@ -3,23 +3,20 @@
 import sys
 import os
 sys.path.append(os.path.realpath("."))
-
 import unittest
-import lt_proc
-import lt_trim
-import lt_print
-import lt_comp
-import lt_append
 
 os.environ['LTTOOLBOX_PATH'] = '../lttoolbox'
 if len(sys.argv) > 1:
-	os.environ['LTTOOLBOX_PATH'] = sys.argv[1]
+    os.environ['LTTOOLBOX_PATH'] = sys.argv[1]
+
+modules = ['lt_proc', 'lt_trim', 'lt_print', 'lt_comp', 'lt_append',
+           'lt_paradigm', 'lt_expand', 'lt_apply_acx', 'lt_compose']
 
 if __name__ == "__main__":
     os.chdir(os.path.dirname(__file__))
     failures = 0
-    for module in [lt_trim, lt_proc, lt_print, lt_comp, lt_append]:
-        suite = unittest.TestLoader().loadTestsFromModule(module)
+    for module in modules:
+        suite = unittest.TestLoader().loadTestsFromName(module)
         res = unittest.TextTestRunner(verbosity = 2).run(suite)
         failures += len(res.failures)
     sys.exit(min(failures, 255))
